from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class CuentaExtranjeraType(XElement):
    def __init__(
            self,
            numero_cuenta: str,
            nombre_banco: str,
    ): 
        """
        
        :param numero_cuenta: 
        :param nombre_banco: 
        """
        
        super().__init__({
            'NumeroCuenta': numero_cuenta,
            'NombreBanco': nombre_banco,
        })
        

class CuentaNacionalType(XElement):
    def __init__(
            self,
            clabe_destino: str,
            clave_institucion_financiera: int,
    ): 
        """
        
        :param clabe_destino: 
        :param clave_institucion_financiera: 
        """
        
        super().__init__({
            'ClabeDestino': clabe_destino,
            'ClaveInstitucionFinanciera': clave_institucion_financiera,
        })
        

class PaisCuentaType(XElement):
    def __init__(
            self,
            nacional: CuentaNacionalType | dict = None,
            extranjero: CuentaExtranjeraType | dict = None,
    ): 
        """
        
        :param nacional: 
        :param extranjero: 
        """
        
        super().__init__({
            'Nacional': nacional,
            'Extranjero': extranjero,
        })
        

class PersonaMoralBasicoType(XElement):
    def __init__(
            self,
            denominacion_razon: str,
    ): 
        """
        
        :param denominacion_razon: 
        """
        
        super().__init__({
            'DenominacionRazon': denominacion_razon,
        })
        

class PersonaFisicaBasicoType(XElement):
    def __init__(
            self,
            nombre: str,
            apellido_paterno: str,
            apellido_materno: str,
    ): 
        """
        
        :param nombre: 
        :param apellido_paterno: 
        :param apellido_materno: 
        """
        
        super().__init__({
            'Nombre': nombre,
            'ApellidoPaterno': apellido_paterno,
            'ApellidoMaterno': apellido_materno,
        })
        

class TipoPersonaBasicoType(XElement):
    def __init__(
            self,
            persona_fisica: PersonaFisicaBasicoType | dict = None,
            persona_moral: PersonaMoralBasicoType | dict = None,
    ): 
        """
        
        :param persona_fisica: 
        :param persona_moral: 
        """
        
        super().__init__({
            'PersonaFisica': persona_fisica,
            'PersonaMoral': persona_moral,
        })
        

class PersonaCuentaType(XElement):
    def __init__(
            self,
            tipo_persona: TipoPersonaBasicoType | dict,
            nacionalidad_cuenta: PaisCuentaType | dict,
    ): 
        """
        
        :param tipo_persona: 
        :param nacionalidad_cuenta: 
        """
        
        super().__init__({
            'TipoPersona': tipo_persona,
            'NacionalidadCuenta': nacionalidad_cuenta,
        })
        

class ActivoVirtualCompuestoType(XElement):
    def __init__(
            self,
            activo_virtual: ActivoVirtualSimpleType | dict,
            monto_operacion_mn: str,
    ): 
        """
        
        :param activo_virtual: 
        :param monto_operacion_mn: 
        """
        
        super().__init__({
            'ActivoVirtual': activo_virtual,
            'MontoOperacionMn': monto_operacion_mn,
        })
        

class ActivoVirtualSimpleType(XElement):
    def __init__(
            self,
            activo_virtual_operado: int,
            tipo_cambio_mn: str,
            cantidad_activo_virtual: str,
            descripcion_activo_virtual: str = None,
    ): 
        """
        
        :param activo_virtual_operado: 
        :param tipo_cambio_mn: 
        :param cantidad_activo_virtual: 
        :param descripcion_activo_virtual: 
        """
        
        super().__init__({
            'ActivoVirtualOperado': activo_virtual_operado,
            'TipoCambioMn': tipo_cambio_mn,
            'CantidadActivoVirtual': cantidad_activo_virtual,
            'DescripcionActivoVirtual': descripcion_activo_virtual,
        })
        

class DepositoType(XElement):
    def __init__(
            self,
            fecha_hora_operacion: str,
            instrumento_monetario: str,
            moneda_operacion: int,
            monto_operacion: str,
            datos_ordenante: PersonaCuentaType | dict,
    ): 
        """
        
        :param fecha_hora_operacion: 
        :param instrumento_monetario: 
        :param moneda_operacion: 
        :param monto_operacion: 
        :param datos_ordenante: 
        """
        
        super().__init__({
            'FechaHoraOperacion': fecha_hora_operacion,
            'InstrumentoMonetario': instrumento_monetario,
            'MonedaOperacion': moneda_operacion,
            'MontoOperacion': monto_operacion,
            'DatosOrdenante': datos_ordenante,
        })
        

class RetiroType(XElement):
    def __init__(
            self,
            fecha_hora_operacion: str,
            instrumento_monetario: str,
            moneda_operacion: int,
            monto_operacion: str,
            datos_beneficiario: PersonaCuentaType | dict,
    ): 
        """
        
        :param fecha_hora_operacion: 
        :param instrumento_monetario: 
        :param moneda_operacion: 
        :param monto_operacion: 
        :param datos_beneficiario: 
        """
        
        super().__init__({
            'FechaHoraOperacion': fecha_hora_operacion,
            'InstrumentoMonetario': instrumento_monetario,
            'MonedaOperacion': moneda_operacion,
            'MontoOperacion': monto_operacion,
            'DatosBeneficiario': datos_beneficiario,
        })
        

class RecepcionType(XElement):
    def __init__(
            self,
            fecha_hora_operacion: str,
            monto_operacion_mn: str,
            activo_virtual: ActivoVirtualSimpleType | dict,
            hash_operacion: str,
    ): 
        """
        
        :param fecha_hora_operacion: 
        :param monto_operacion_mn: 
        :param activo_virtual: 
        :param hash_operacion: 
        """
        
        super().__init__({
            'FechaHoraOperacion': fecha_hora_operacion,
            'MontoOperacionMn': monto_operacion_mn,
            'ActivoVirtual': activo_virtual,
            'HashOperacion': hash_operacion,
        })
        

class EnvioType(XElement):
    def __init__(
            self,
            fecha_hora_operacion: str,
            monto_operacion_mn: str,
            activo_virtual: ActivoVirtualSimpleType | dict,
            hash_operacion: str,
    ): 
        """
        
        :param fecha_hora_operacion: 
        :param monto_operacion_mn: 
        :param activo_virtual: 
        :param hash_operacion: 
        """
        
        super().__init__({
            'FechaHoraOperacion': fecha_hora_operacion,
            'MontoOperacionMn': monto_operacion_mn,
            'ActivoVirtual': activo_virtual,
            'HashOperacion': hash_operacion,
        })
        

class IntercambioType(XElement):
    def __init__(
            self,
            fecha_hora_operacion: str,
            activo_virtual_enviado: ActivoVirtualCompuestoType | dict,
            activo_virtual_recibido: ActivoVirtualCompuestoType | dict,
            hash_operacion: str,
    ): 
        """
        
        :param fecha_hora_operacion: 
        :param activo_virtual_enviado: 
        :param activo_virtual_recibido: 
        :param hash_operacion: 
        """
        
        super().__init__({
            'FechaHoraOperacion': fecha_hora_operacion,
            'ActivoVirtualEnviado': activo_virtual_enviado,
            'ActivoVirtualRecibido': activo_virtual_recibido,
            'HashOperacion': hash_operacion,
        })
        

class VentaType(XElement):
    def __init__(
            self,
            fecha_hora_operacion: str,
            moneda_operacion: int,
            monto_operacion: str,
            activo_virtual: ActivoVirtualSimpleType | dict,
            hash_operacion: str,
    ): 
        """
        
        :param fecha_hora_operacion: 
        :param moneda_operacion: 
        :param monto_operacion: 
        :param activo_virtual: 
        :param hash_operacion: 
        """
        
        super().__init__({
            'FechaHoraOperacion': fecha_hora_operacion,
            'MonedaOperacion': moneda_operacion,
            'MontoOperacion': monto_operacion,
            'ActivoVirtual': activo_virtual,
            'HashOperacion': hash_operacion,
        })
        

class CompraType(XElement):
    def __init__(
            self,
            fecha_hora_operacion: str,
            moneda_operacion: int,
            monto_operacion: str,
            activo_virtual: ActivoVirtualSimpleType | dict,
            hash_operacion: str,
    ): 
        """
        
        :param fecha_hora_operacion: 
        :param moneda_operacion: 
        :param monto_operacion: 
        :param activo_virtual: 
        :param hash_operacion: 
        """
        
        super().__init__({
            'FechaHoraOperacion': fecha_hora_operacion,
            'MonedaOperacion': moneda_operacion,
            'MontoOperacion': monto_operacion,
            'ActivoVirtual': activo_virtual,
            'HashOperacion': hash_operacion,
        })
        

class FondosDepositadosType(XElement):
    def __init__(
            self,
            deposito: DepositoType | dict | Sequence[DepositoType | dict],
    ): 
        """
        
        :param deposito: 
        """
        
        super().__init__({
            'Deposito': deposito,
        })
        

class FondosRetiradosType(XElement):
    def __init__(
            self,
            retiro: RetiroType | dict | Sequence[RetiroType | dict],
    ): 
        """
        
        :param retiro: 
        """
        
        super().__init__({
            'Retiro': retiro,
        })
        

class OperacionesFondosType(XElement):
    def __init__(
            self,
            fondos_retirados: FondosRetiradosType | dict = None,
            fondos_depositados: FondosDepositadosType | dict = None,
    ): 
        """
        
        :param fondos_retirados: 
        :param fondos_depositados: 
        """
        
        super().__init__({
            'FondosRetirados': fondos_retirados,
            'FondosDepositados': fondos_depositados,
        })
        

class TransferenciasRecibidasType(XElement):
    def __init__(
            self,
            recepcion: RecepcionType | dict | Sequence[RecepcionType | dict],
    ): 
        """
        
        :param recepcion: 
        """
        
        super().__init__({
            'Recepcion': recepcion,
        })
        

class TransferenciasEnviadasType(XElement):
    def __init__(
            self,
            envio: EnvioType | dict | Sequence[EnvioType | dict],
    ): 
        """
        
        :param envio: 
        """
        
        super().__init__({
            'Envio': envio,
        })
        

class OperacionesTransferenciaType(XElement):
    def __init__(
            self,
            transferencias_enviadas: TransferenciasEnviadasType | dict = None,
            transferencias_recibidas: TransferenciasRecibidasType | dict = None,
    ): 
        """
        
        :param transferencias_enviadas: 
        :param transferencias_recibidas: 
        """
        
        super().__init__({
            'TransferenciasEnviadas': transferencias_enviadas,
            'TransferenciasRecibidas': transferencias_recibidas,
        })
        

class OperacionesIntercambioType(XElement):
    def __init__(
            self,
            intercambio: IntercambioType | dict | Sequence[IntercambioType | dict],
    ): 
        """
        
        :param intercambio: 
        """
        
        super().__init__({
            'Intercambio': intercambio,
        })
        

class OperacionesVentaType(XElement):
    def __init__(
            self,
            venta: VentaType | dict | Sequence[VentaType | dict],
    ): 
        """
        
        :param venta: 
        """
        
        super().__init__({
            'Venta': venta,
        })
        

class OperacionesCompraType(XElement):
    def __init__(
            self,
            compra: CompraType | dict | Sequence[CompraType | dict],
    ): 
        """
        
        :param compra: 
        """
        
        super().__init__({
            'Compra': compra,
        })
        

class DetalleOperacionesType(XElement):
    def __init__(
            self,
            operaciones_compra: OperacionesCompraType | dict = None,
            operaciones_venta: OperacionesVentaType | dict = None,
            operaciones_intercambio: OperacionesIntercambioType | dict = None,
            operaciones_transferencia: OperacionesTransferenciaType | dict = None,
            operaciones_fondos: OperacionesFondosType | dict = None,
    ): 
        """
        
        :param operaciones_compra: 
        :param operaciones_venta: 
        :param operaciones_intercambio: 
        :param operaciones_transferencia: 
        :param operaciones_fondos: 
        """
        
        super().__init__({
            'OperacionesCompra': operaciones_compra,
            'OperacionesVenta': operaciones_venta,
            'OperacionesIntercambio': operaciones_intercambio,
            'OperacionesTransferencia': operaciones_transferencia,
            'OperacionesFondos': operaciones_fondos,
        })
        

class FideicomisoSimpleType(XElement):
    def __init__(
            self,
            denominacion_razon: str,
            rfc: str = None,
            identificador_fideicomiso: str = None,
    ): 
        """
        
        :param denominacion_razon: 
        :param rfc: 
        :param identificador_fideicomiso: 
        """
        
        super().__init__({
            'DenominacionRazon': denominacion_razon,
            'Rfc': rfc,
            'IdentificadorFideicomiso': identificador_fideicomiso,
        })
        

class PersonaMoralSimpleType(XElement):
    def __init__(
            self,
            denominacion_razon: str,
            pais_nacionalidad: str,
            fecha_constitucion: date = None,
            rfc: str = None,
    ): 
        """
        
        :param denominacion_razon: 
        :param pais_nacionalidad: 
        :param fecha_constitucion: 
        :param rfc: 
        """
        
        super().__init__({
            'DenominacionRazon': denominacion_razon,
            'PaisNacionalidad': pais_nacionalidad,
            'FechaConstitucion': fecha_constitucion,
            'Rfc': rfc,
        })
        

class PersonaFisicaSimpleType(XElement):
    def __init__(
            self,
            nombre: str,
            apellido_paterno: str,
            apellido_materno: str,
            pais_nacionalidad: str,
            fecha_nacimiento: date = None,
            rfc: str = None,
            curp: str = None,
    ): 
        """
        
        :param nombre: 
        :param apellido_paterno: 
        :param apellido_materno: 
        :param pais_nacionalidad: 
        :param fecha_nacimiento: 
        :param rfc: 
        :param curp: 
        """
        
        super().__init__({
            'Nombre': nombre,
            'ApellidoPaterno': apellido_paterno,
            'ApellidoMaterno': apellido_materno,
            'PaisNacionalidad': pais_nacionalidad,
            'FechaNacimiento': fecha_nacimiento,
            'Rfc': rfc,
            'Curp': curp,
        })
        

class TipoPersonaSimpleType(XElement):
    def __init__(
            self,
            persona_fisica: PersonaFisicaSimpleType | dict = None,
            persona_moral: PersonaMoralSimpleType | dict = None,
            fideicomiso: FideicomisoSimpleType | dict = None,
    ): 
        """
        
        :param persona_fisica: 
        :param persona_moral: 
        :param fideicomiso: 
        """
        
        super().__init__({
            'PersonaFisica': persona_fisica,
            'PersonaMoral': persona_moral,
            'Fideicomiso': fideicomiso,
        })
        

class DuenoBeneficiarioType(XElement):
    def __init__(
            self,
            tipo_persona: TipoPersonaSimpleType | dict,
    ): 
        """
        
        :param tipo_persona: 
        """
        
        super().__init__({
            'TipoPersona': tipo_persona,
        })
        

class TelefonoType(XElement):
    def __init__(
            self,
            clave_pais: str,
            numero_telefono: str,
            correo_electronico: str,
    ): 
        """
        
        :param clave_pais: 
        :param numero_telefono: 
        :param correo_electronico: 
        """
        
        super().__init__({
            'ClavePais': clave_pais,
            'NumeroTelefono': numero_telefono,
            'CorreoElectronico': correo_electronico,
        })
        

class ExtranjeroType(XElement):
    def __init__(
            self,
            pais: str,
            estado_provincia: str,
            ciudad_poblacion: str,
            colonia: str,
            calle: str,
            numero_exterior: str,
            codigo_postal: str,
            numero_interior: str = None,
    ): 
        """
        
        :param pais: 
        :param estado_provincia: 
        :param ciudad_poblacion: 
        :param colonia: 
        :param calle: 
        :param numero_exterior: 
        :param codigo_postal: 
        :param numero_interior: 
        """
        
        super().__init__({
            'Pais': pais,
            'EstadoProvincia': estado_provincia,
            'CiudadPoblacion': ciudad_poblacion,
            'Colonia': colonia,
            'Calle': calle,
            'NumeroExterior': numero_exterior,
            'CodigoPostal': codigo_postal,
            'NumeroInterior': numero_interior,
        })
        

class NacionalType(XElement):
    def __init__(
            self,
            colonia: str,
            calle: str,
            numero_exterior: str,
            codigo_postal: str,
            numero_interior: str = None,
    ): 
        """
        
        :param colonia: 
        :param calle: 
        :param numero_exterior: 
        :param codigo_postal: 
        :param numero_interior: 
        """
        
        super().__init__({
            'Colonia': colonia,
            'Calle': calle,
            'NumeroExterior': numero_exterior,
            'CodigoPostal': codigo_postal,
            'NumeroInterior': numero_interior,
        })
        

class TipoDomicilioType(XElement):
    def __init__(
            self,
            nacional: NacionalType | dict = None,
            extranjero: ExtranjeroType | dict = None,
    ): 
        """
        
        :param nacional: 
        :param extranjero: 
        """
        
        super().__init__({
            'Nacional': nacional,
            'Extranjero': extranjero,
        })
        

class DocumentoIdentificacionType(XElement):
    def __init__(
            self,
            tipo_identificacion: int,
            numero_identificacion: str,
    ): 
        """
        
        :param tipo_identificacion: 
        :param numero_identificacion: 
        """
        
        super().__init__({
            'TipoIdentificacion': tipo_identificacion,
            'NumeroIdentificacion': numero_identificacion,
        })
        

class RepresentanteApoderadoType(XElement):
    def __init__(
            self,
            nombre: str,
            apellido_paterno: str,
            apellido_materno: str,
            documento_identificacion: DocumentoIdentificacionType | dict,
            fecha_nacimiento: date = None,
            rfc: str = None,
            curp: str = None,
    ): 
        """
        
        :param nombre: 
        :param apellido_paterno: 
        :param apellido_materno: 
        :param documento_identificacion: 
        :param fecha_nacimiento: 
        :param rfc: 
        :param curp: 
        """
        
        super().__init__({
            'Nombre': nombre,
            'ApellidoPaterno': apellido_paterno,
            'ApellidoMaterno': apellido_materno,
            'DocumentoIdentificacion': documento_identificacion,
            'FechaNacimiento': fecha_nacimiento,
            'Rfc': rfc,
            'Curp': curp,
        })
        

class FideicomisoType(XElement):
    def __init__(
            self,
            denominacion_razon: str,
            apoderado_delegado: RepresentanteApoderadoType | dict,
            rfc: str = None,
            identificador_fideicomiso: str = None,
    ): 
        """
        
        :param denominacion_razon: 
        :param apoderado_delegado: 
        :param rfc: 
        :param identificador_fideicomiso: 
        """
        
        super().__init__({
            'DenominacionRazon': denominacion_razon,
            'ApoderadoDelegado': apoderado_delegado,
            'Rfc': rfc,
            'IdentificadorFideicomiso': identificador_fideicomiso,
        })
        

class PersonaMoralType(XElement):
    def __init__(
            self,
            denominacion_razon: str,
            pais_nacionalidad: str,
            giro_mercantil: str,
            representante_apoderado: RepresentanteApoderadoType | dict,
            fecha_constitucion: date = None,
            rfc: str = None,
    ): 
        """
        
        :param denominacion_razon: 
        :param pais_nacionalidad: 
        :param giro_mercantil: 
        :param representante_apoderado: 
        :param fecha_constitucion: 
        :param rfc: 
        """
        
        super().__init__({
            'DenominacionRazon': denominacion_razon,
            'PaisNacionalidad': pais_nacionalidad,
            'GiroMercantil': giro_mercantil,
            'RepresentanteApoderado': representante_apoderado,
            'FechaConstitucion': fecha_constitucion,
            'Rfc': rfc,
        })
        

class PersonaFisicaType(XElement):
    def __init__(
            self,
            nombre: str,
            apellido_paterno: str,
            apellido_materno: str,
            pais_nacionalidad: str,
            actividad_economica: str,
            documento_identificacion: DocumentoIdentificacionType | dict,
            fecha_nacimiento: date = None,
            rfc: str = None,
            curp: str = None,
    ): 
        """
        
        :param nombre: 
        :param apellido_paterno: 
        :param apellido_materno: 
        :param pais_nacionalidad: 
        :param actividad_economica: 
        :param documento_identificacion: 
        :param fecha_nacimiento: 
        :param rfc: 
        :param curp: 
        """
        
        super().__init__({
            'Nombre': nombre,
            'ApellidoPaterno': apellido_paterno,
            'ApellidoMaterno': apellido_materno,
            'PaisNacionalidad': pais_nacionalidad,
            'ActividadEconomica': actividad_economica,
            'DocumentoIdentificacion': documento_identificacion,
            'FechaNacimiento': fecha_nacimiento,
            'Rfc': rfc,
            'Curp': curp,
        })
        

class TipoPersonaType(XElement):
    def __init__(
            self,
            persona_fisica: PersonaFisicaType | dict = None,
            persona_moral: PersonaMoralType | dict = None,
            fideicomiso: FideicomisoType | dict = None,
    ): 
        """
        
        :param persona_fisica: 
        :param persona_moral: 
        :param fideicomiso: 
        """
        
        super().__init__({
            'PersonaFisica': persona_fisica,
            'PersonaMoral': persona_moral,
            'Fideicomiso': fideicomiso,
        })
        

class DatosCuentaPlataformaType(XElement):
    def __init__(
            self,
            id_usuario: str,
            cuenta_relacionada: str,
            moneda_cuenta: int,
            clabe_interbancaria: str = None,
    ): 
        """
        
        :param id_usuario: 
        :param cuenta_relacionada: 
        :param moneda_cuenta: 
        :param clabe_interbancaria: 
        """
        
        super().__init__({
            'IdUsuario': id_usuario,
            'CuentaRelacionada': cuenta_relacionada,
            'MonedaCuenta': moneda_cuenta,
            'ClabeInterbancaria': clabe_interbancaria,
        })
        

class PersonaAvisoType(XElement):
    def __init__(
            self,
            datos_cuenta_plataforma: DatosCuentaPlataformaType | dict,
            tipo_persona: TipoPersonaType | dict,
            tipo_domicilio: TipoDomicilioType | dict = None,
            telefono: TelefonoType | dict = None,
    ): 
        """
        
        :param datos_cuenta_plataforma: 
        :param tipo_persona: 
        :param tipo_domicilio: 
        :param telefono: 
        """
        
        super().__init__({
            'DatosCuentaPlataforma': datos_cuenta_plataforma,
            'TipoPersona': tipo_persona,
            'TipoDomicilio': tipo_domicilio,
            'Telefono': telefono,
        })
        

class OperacionesPersonaType(XElement):
    def __init__(
            self,
            persona_aviso: PersonaAvisoType | dict,
            detalle_operaciones: DetalleOperacionesType | dict,
            dueno_beneficiario: DuenoBeneficiarioType | dict | Sequence[DuenoBeneficiarioType | dict] = None,
    ): 
        """
        
        :param persona_aviso: 
        :param detalle_operaciones: 
        :param dueno_beneficiario: 
        """
        
        super().__init__({
            'PersonaAviso': persona_aviso,
            'DetalleOperaciones': detalle_operaciones,
            'DuenoBeneficiario': dueno_beneficiario,
        })
        

class AlertaType(XElement):
    def __init__(
            self,
            tipo_alerta: str,
            descripcion_alerta: str = None,
    ): 
        """
        
        :param tipo_alerta: 
        :param descripcion_alerta: 
        """
        
        super().__init__({
            'TipoAlerta': tipo_alerta,
            'DescripcionAlerta': descripcion_alerta,
        })
        

class ModificatorioType(XElement):
    def __init__(
            self,
            folio_modificacion: str,
            descripcion_modificacion: str,
    ): 
        """
        
        :param folio_modificacion: 
        :param descripcion_modificacion: 
        """
        
        super().__init__({
            'FolioModificacion': folio_modificacion,
            'DescripcionModificacion': descripcion_modificacion,
        })
        

class AvisoType(XElement):
    def __init__(
            self,
            referencia_aviso: str,
            prioridad: str,
            alerta: AlertaType | dict,
            operaciones_persona: OperacionesPersonaType | dict,
            modificatorio: ModificatorioType | dict = None,
    ): 
        """
        
        :param referencia_aviso: 
        :param prioridad: 
        :param alerta: 
        :param operaciones_persona: 
        :param modificatorio: 
        """
        
        super().__init__({
            'ReferenciaAviso': referencia_aviso,
            'Prioridad': prioridad,
            'Alerta': alerta,
            'OperacionesPersona': operaciones_persona,
            'Modificatorio': modificatorio,
        })
        

class SujetoObligadoType(XElement):
    def __init__(
            self,
            clave_sujeto_obligado: str,
            clave_actividad: str,
            dominio_plataforma: str,
            clave_entidad_colegiada: str = None,
            exento: str = None,
    ): 
        """
        
        :param clave_sujeto_obligado: 
        :param clave_actividad: 
        :param dominio_plataforma: 
        :param clave_entidad_colegiada: 
        :param exento: 
        """
        
        super().__init__({
            'ClaveSujetoObligado': clave_sujeto_obligado,
            'ClaveActividad': clave_actividad,
            'DominioPlataforma': dominio_plataforma,
            'ClaveEntidadColegiada': clave_entidad_colegiada,
            'Exento': exento,
        })
        

class InformeType(XElement):
    def __init__(
            self,
            mes_reportado: str,
            sujeto_obligado: SujetoObligadoType | dict,
            aviso: AvisoType | dict | Sequence[AvisoType | dict] = None,
    ): 
        """
        
        :param mes_reportado: 
        :param sujeto_obligado: 
        :param aviso: 
        """
        
        super().__init__({
            'MesReportado': mes_reportado,
            'SujetoObligado': sujeto_obligado,
            'Aviso': aviso,
        })
        

class ArchivoType(XElement):
    def __init__(
            self,
            informe: InformeType | dict | Sequence[InformeType | dict],
    ): 
        """
        
        :param informe: 
        """
        
        super().__init__({
            'Informe': informe,
        })
        

class Archivo(ArchivoType):
    tag = '{http://www.uif.shcp.gob.mx/recepcion/avi}archivo'




from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class Intereses(CFDI):
    """
    Complemento para expresar los intereses obtenidos por rendimiento en inversiones
    """
    tag = '{http://www.sat.gob.mx/esquemas/retencionpago/1/intereses}Intereses'
    version = '1.0'
    
    def __init__(
            self,
            sist_financiero: str,
            retiro_aoresret_int: str,
            oper_financ_derivad: str,
            mont_int_nominal: Decimal | int,
            mont_int_real: Decimal | int,
            perdida: Decimal | int,
    ): 
        """
        Complemento para expresar los intereses obtenidos por rendimiento en inversiones
        
        :param sist_financiero: Atributo requerido para expresar si los interés obtenidos en el periodo o ejercicio provienen del sistema financiero
        :param retiro_aoresret_int: Atributo requerido para expresar si los intereses obtenidos fueron retirados en el periodo o ejercicio
        :param oper_financ_derivad: Atributo requerido para expresar si los intereses obtenidos corresponden a operaciones financieras derivadas.
        :param mont_int_nominal: Atributo requerido para expresar el importe del interés Nóminal obtenido en un periodo o ejercicio
        :param mont_int_real: Atributo requerido para expresar el monto de los intereses reales (diferencia que se obtiene restando al tipo de interés nominal y la tasa de inflación del periodo o ejercicio )
        :param perdida: Atributo requerido para expresar la pérdida por los intereses obtenidos en el periodo o ejercicio
        """
        
        super().__init__({
            'Version': self.version,
            'SistFinanciero': sist_financiero,
            'RetiroAORESRetInt': retiro_aoresret_int,
            'OperFinancDerivad': oper_financ_derivad,
            'MontIntNominal': mont_int_nominal,
            'MontIntReal': mont_int_real,
            'Perdida': perdida,
        })
        




import glob
import logging
import sys
from typing import List, Optional

from .. import CFDI, CFDIError

logging.basicConfig(level=logging.INFO)
logging.getLogger("weasyprint").setLevel(logging.ERROR)
logging.getLogger("fontTools").setLevel(logging.ERROR)

logger = logging.getLogger(__name__)


def main(args: Optional[List[str]] = None) -> int:
    if args is None:
        args = sys.argv[1:]

    try:
        cmd_name = args[0]
    except IndexError:
        cmd_name = "help"

    match cmd_name:
        case "pdf":
            try:
                file = args[1]
            except IndexError:
                sys.stderr.write("File argument is missing, example: cfdi pdf myfile.xml")
                sys.exit(1)

            for file in glob.iglob(file):
                cfdi = CFDI.from_file(filename=file)
                target = (file[:-4] if file.endswith(".xml") else file) + ".pdf"
                cfdi.pdf_write(target=target)
                logger.info(f"PDF created: '{file}'")

        case "html":
            try:
                file = args[1]
            except IndexError:
                sys.stderr.write("File argument is missing, example: cfdi pdf myfile.xml")
                sys.exit(1)

            for file in glob.iglob(file):
                cfdi = CFDI.from_file(filename=file)
                target = (file[:-4] if file.endswith(".xml") else file) + ".html"
                cfdi.html_write(target=target)
                logger.info(f"HTML created: '{file}'")

        case "json":
            try:
                file = args[1]
            except IndexError:
                sys.stderr.write("File argument is missing, example: cfdi pdf myfile.xml")
                sys.exit(1)

            for file in glob.iglob(file):
                cfdi = CFDI.from_file(filename=file)
                target = (file[:-4] if file.endswith(".xml") else file) + ".json"
                cfdi.json_write(target=target, pretty_print=True)
                logger.info(f"JSON created: '{file}'")

        case "help":
            sys.stdout.write("Looks like you need some help\n")
            sys.stdout.write("Commands:\n")
            sys.stdout.write(" pdf       creates a pdf file\n")
            sys.stdout.write(" html      creates a html file\n")
            sys.stdout.write(" json      creates a json file\n")

        case _:
            raise CFDIError("Invalid Command")

    return 0



from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class DatosLiquidacionType(XElement):
    def __init__(
            self,
            fecha_pago: date,
            forma_pago: str,
            instrumento_monetario: str,
            moneda: str,
            monto_operacion: str,
    ): 
        """
        
        :param fecha_pago: 
        :param forma_pago: 
        :param instrumento_monetario: 
        :param moneda: 
        :param monto_operacion: 
        """
        
        super().__init__({
            'FechaPago': fecha_pago,
            'FormaPago': forma_pago,
            'InstrumentoMonetario': instrumento_monetario,
            'Moneda': moneda,
            'MontoOperacion': monto_operacion,
        })
        

class CaracteristicasType(XElement):
    def __init__(
            self,
            fecha_inicio: date,
            fecha_termino: date,
            tipo_inmueble: str,
            valor_referencia: str,
            colonia: str,
            calle: str,
            numero_exterior: str,
            codigo_postal: str,
            folio_real: str,
            numero_interior: str = None,
    ): 
        """
        
        :param fecha_inicio: 
        :param fecha_termino: 
        :param tipo_inmueble: 
        :param valor_referencia: 
        :param colonia: 
        :param calle: 
        :param numero_exterior: 
        :param codigo_postal: 
        :param folio_real: 
        :param numero_interior: 
        """
        
        super().__init__({
            'FechaInicio': fecha_inicio,
            'FechaTermino': fecha_termino,
            'TipoInmueble': tipo_inmueble,
            'ValorReferencia': valor_referencia,
            'Colonia': colonia,
            'Calle': calle,
            'NumeroExterior': numero_exterior,
            'CodigoPostal': codigo_postal,
            'FolioReal': folio_real,
            'NumeroInterior': numero_interior,
        })
        

class DatosOperacionType(XElement):
    def __init__(
            self,
            fecha_operacion: date,
            tipo_operacion: str,
            caracteristicas: CaracteristicasType | dict | Sequence[CaracteristicasType | dict],
            datos_liquidacion: DatosLiquidacionType | dict | Sequence[DatosLiquidacionType | dict],
    ): 
        """
        
        :param fecha_operacion: 
        :param tipo_operacion: 
        :param caracteristicas: 
        :param datos_liquidacion: 
        """
        
        super().__init__({
            'FechaOperacion': fecha_operacion,
            'TipoOperacion': tipo_operacion,
            'Caracteristicas': caracteristicas,
            'DatosLiquidacion': datos_liquidacion,
        })
        

class DetalleOperacionesType(XElement):
    def __init__(
            self,
            datos_operacion: DatosOperacionType | dict | Sequence[DatosOperacionType | dict],
    ): 
        """
        
        :param datos_operacion: 
        """
        
        super().__init__({
            'DatosOperacion': datos_operacion,
        })
        

class FideicomisoSimpleType(XElement):
    def __init__(
            self,
            denominacion_razon: str,
            rfc: str = None,
            identificador_fideicomiso: str = None,
    ): 
        """
        
        :param denominacion_razon: 
        :param rfc: 
        :param identificador_fideicomiso: 
        """
        
        super().__init__({
            'DenominacionRazon': denominacion_razon,
            'Rfc': rfc,
            'IdentificadorFideicomiso': identificador_fideicomiso,
        })
        

class PersonaMoralSimpleType(XElement):
    def __init__(
            self,
            denominacion_razon: str,
            fecha_constitucion: date = None,
            rfc: str = None,
            pais_nacionalidad: str = None,
    ): 
        """
        
        :param denominacion_razon: 
        :param fecha_constitucion: 
        :param rfc: 
        :param pais_nacionalidad: 
        """
        
        super().__init__({
            'DenominacionRazon': denominacion_razon,
            'FechaConstitucion': fecha_constitucion,
            'Rfc': rfc,
            'PaisNacionalidad': pais_nacionalidad,
        })
        

class PersonaFisicaSimpleType(XElement):
    def __init__(
            self,
            nombre: str,
            apellido_paterno: str,
            apellido_materno: str,
            fecha_nacimiento: date = None,
            rfc: str = None,
            curp: str = None,
            pais_nacionalidad: str = None,
    ): 
        """
        
        :param nombre: 
        :param apellido_paterno: 
        :param apellido_materno: 
        :param fecha_nacimiento: 
        :param rfc: 
        :param curp: 
        :param pais_nacionalidad: 
        """
        
        super().__init__({
            'Nombre': nombre,
            'ApellidoPaterno': apellido_paterno,
            'ApellidoMaterno': apellido_materno,
            'FechaNacimiento': fecha_nacimiento,
            'Rfc': rfc,
            'Curp': curp,
            'PaisNacionalidad': pais_nacionalidad,
        })
        

class TipoPersonaSimpleType(XElement):
    def __init__(
            self,
            persona_fisica: PersonaFisicaSimpleType | dict = None,
            persona_moral: PersonaMoralSimpleType | dict = None,
            fideicomiso: FideicomisoSimpleType | dict = None,
    ): 
        """
        
        :param persona_fisica: 
        :param persona_moral: 
        :param fideicomiso: 
        """
        
        super().__init__({
            'PersonaFisica': persona_fisica,
            'PersonaMoral': persona_moral,
            'Fideicomiso': fideicomiso,
        })
        

class DuenoBeneficiarioType(XElement):
    def __init__(
            self,
            tipo_persona: TipoPersonaSimpleType | dict,
    ): 
        """
        
        :param tipo_persona: 
        """
        
        super().__init__({
            'TipoPersona': tipo_persona,
        })
        

class TelefonoType(XElement):
    def __init__(
            self,
            clave_pais: str = None,
            numero_telefono: str = None,
            correo_electronico: str = None,
    ): 
        """
        
        :param clave_pais: 
        :param numero_telefono: 
        :param correo_electronico: 
        """
        
        super().__init__({
            'ClavePais': clave_pais,
            'NumeroTelefono': numero_telefono,
            'CorreoElectronico': correo_electronico,
        })
        

class ExtranjeroType(XElement):
    def __init__(
            self,
            pais: str,
            estado_provincia: str,
            ciudad_poblacion: str,
            colonia: str,
            calle: str,
            numero_exterior: str,
            codigo_postal: str,
            numero_interior: str = None,
    ): 
        """
        
        :param pais: 
        :param estado_provincia: 
        :param ciudad_poblacion: 
        :param colonia: 
        :param calle: 
        :param numero_exterior: 
        :param codigo_postal: 
        :param numero_interior: 
        """
        
        super().__init__({
            'Pais': pais,
            'EstadoProvincia': estado_provincia,
            'CiudadPoblacion': ciudad_poblacion,
            'Colonia': colonia,
            'Calle': calle,
            'NumeroExterior': numero_exterior,
            'CodigoPostal': codigo_postal,
            'NumeroInterior': numero_interior,
        })
        

class NacionalType(XElement):
    def __init__(
            self,
            colonia: str,
            calle: str,
            numero_exterior: str,
            codigo_postal: str,
            numero_interior: str = None,
    ): 
        """
        
        :param colonia: 
        :param calle: 
        :param numero_exterior: 
        :param codigo_postal: 
        :param numero_interior: 
        """
        
        super().__init__({
            'Colonia': colonia,
            'Calle': calle,
            'NumeroExterior': numero_exterior,
            'CodigoPostal': codigo_postal,
            'NumeroInterior': numero_interior,
        })
        

class TipoDomicilioType(XElement):
    def __init__(
            self,
            nacional: NacionalType | dict = None,
            extranjero: ExtranjeroType | dict = None,
    ): 
        """
        
        :param nacional: 
        :param extranjero: 
        """
        
        super().__init__({
            'Nacional': nacional,
            'Extranjero': extranjero,
        })
        

class RepresentanteApoderadoType(XElement):
    def __init__(
            self,
            nombre: str,
            apellido_paterno: str,
            apellido_materno: str,
            fecha_nacimiento: date = None,
            rfc: str = None,
            curp: str = None,
    ): 
        """
        
        :param nombre: 
        :param apellido_paterno: 
        :param apellido_materno: 
        :param fecha_nacimiento: 
        :param rfc: 
        :param curp: 
        """
        
        super().__init__({
            'Nombre': nombre,
            'ApellidoPaterno': apellido_paterno,
            'ApellidoMaterno': apellido_materno,
            'FechaNacimiento': fecha_nacimiento,
            'Rfc': rfc,
            'Curp': curp,
        })
        

class FideicomisoType(XElement):
    def __init__(
            self,
            denominacion_razon: str,
            apoderado_delegado: RepresentanteApoderadoType | dict,
            rfc: str = None,
            identificador_fideicomiso: str = None,
    ): 
        """
        
        :param denominacion_razon: 
        :param apoderado_delegado: 
        :param rfc: 
        :param identificador_fideicomiso: 
        """
        
        super().__init__({
            'DenominacionRazon': denominacion_razon,
            'ApoderadoDelegado': apoderado_delegado,
            'Rfc': rfc,
            'IdentificadorFideicomiso': identificador_fideicomiso,
        })
        

class PersonaMoralType(XElement):
    def __init__(
            self,
            denominacion_razon: str,
            pais_nacionalidad: str,
            giro_mercantil: str,
            representante_apoderado: RepresentanteApoderadoType | dict,
            fecha_constitucion: date = None,
            rfc: str = None,
    ): 
        """
        
        :param denominacion_razon: 
        :param pais_nacionalidad: 
        :param giro_mercantil: 
        :param representante_apoderado: 
        :param fecha_constitucion: 
        :param rfc: 
        """
        
        super().__init__({
            'DenominacionRazon': denominacion_razon,
            'PaisNacionalidad': pais_nacionalidad,
            'GiroMercantil': giro_mercantil,
            'RepresentanteApoderado': representante_apoderado,
            'FechaConstitucion': fecha_constitucion,
            'Rfc': rfc,
        })
        

class PersonaFisicaType(XElement):
    def __init__(
            self,
            nombre: str,
            apellido_paterno: str,
            apellido_materno: str,
            pais_nacionalidad: str,
            actividad_economica: str,
            fecha_nacimiento: date = None,
            rfc: str = None,
            curp: str = None,
    ): 
        """
        
        :param nombre: 
        :param apellido_paterno: 
        :param apellido_materno: 
        :param pais_nacionalidad: 
        :param actividad_economica: 
        :param fecha_nacimiento: 
        :param rfc: 
        :param curp: 
        """
        
        super().__init__({
            'Nombre': nombre,
            'ApellidoPaterno': apellido_paterno,
            'ApellidoMaterno': apellido_materno,
            'PaisNacionalidad': pais_nacionalidad,
            'ActividadEconomica': actividad_economica,
            'FechaNacimiento': fecha_nacimiento,
            'Rfc': rfc,
            'Curp': curp,
        })
        

class TipoPersonaType(XElement):
    def __init__(
            self,
            persona_fisica: PersonaFisicaType | dict = None,
            persona_moral: PersonaMoralType | dict = None,
            fideicomiso: FideicomisoType | dict = None,
    ): 
        """
        
        :param persona_fisica: 
        :param persona_moral: 
        :param fideicomiso: 
        """
        
        super().__init__({
            'PersonaFisica': persona_fisica,
            'PersonaMoral': persona_moral,
            'Fideicomiso': fideicomiso,
        })
        

class PersonaAvisoType(XElement):
    def __init__(
            self,
            tipo_persona: TipoPersonaType | dict,
            tipo_domicilio: TipoDomicilioType | dict = None,
            telefono: TelefonoType | dict = None,
    ): 
        """
        
        :param tipo_persona: 
        :param tipo_domicilio: 
        :param telefono: 
        """
        
        super().__init__({
            'TipoPersona': tipo_persona,
            'TipoDomicilio': tipo_domicilio,
            'Telefono': telefono,
        })
        

class AlertaType(XElement):
    def __init__(
            self,
            tipo_alerta: str,
            descripcion_alerta: str = None,
    ): 
        """
        
        :param tipo_alerta: 
        :param descripcion_alerta: 
        """
        
        super().__init__({
            'TipoAlerta': tipo_alerta,
            'DescripcionAlerta': descripcion_alerta,
        })
        

class ModificatorioType(XElement):
    def __init__(
            self,
            folio_modificacion: str,
            descripcion_modificacion: str,
    ): 
        """
        
        :param folio_modificacion: 
        :param descripcion_modificacion: 
        """
        
        super().__init__({
            'FolioModificacion': folio_modificacion,
            'DescripcionModificacion': descripcion_modificacion,
        })
        

class AvisoType(XElement):
    def __init__(
            self,
            referencia_aviso: str,
            prioridad: str,
            alerta: AlertaType | dict,
            persona_aviso: PersonaAvisoType | dict | Sequence[PersonaAvisoType | dict],
            detalle_operaciones: DetalleOperacionesType | dict,
            modificatorio: ModificatorioType | dict = None,
            dueno_beneficiario: DuenoBeneficiarioType | dict | Sequence[DuenoBeneficiarioType | dict] = None,
    ): 
        """
        
        :param referencia_aviso: 
        :param prioridad: 
        :param alerta: 
        :param persona_aviso: 
        :param detalle_operaciones: 
        :param modificatorio: 
        :param dueno_beneficiario: 
        """
        
        super().__init__({
            'ReferenciaAviso': referencia_aviso,
            'Prioridad': prioridad,
            'Alerta': alerta,
            'PersonaAviso': persona_aviso,
            'DetalleOperaciones': detalle_operaciones,
            'Modificatorio': modificatorio,
            'DuenoBeneficiario': dueno_beneficiario,
        })
        

class SujetoObligadoType(XElement):
    def __init__(
            self,
            clave_sujeto_obligado: str,
            clave_actividad: str,
            clave_entidad_colegiada: str = None,
            exento: str = None,
    ): 
        """
        
        :param clave_sujeto_obligado: 
        :param clave_actividad: 
        :param clave_entidad_colegiada: 
        :param exento: 
        """
        
        super().__init__({
            'ClaveSujetoObligado': clave_sujeto_obligado,
            'ClaveActividad': clave_actividad,
            'ClaveEntidadColegiada': clave_entidad_colegiada,
            'Exento': exento,
        })
        

class InformeType(XElement):
    def __init__(
            self,
            mes_reportado: str,
            sujeto_obligado: SujetoObligadoType | dict,
            aviso: AvisoType | dict | Sequence[AvisoType | dict] = None,
    ): 
        """
        
        :param mes_reportado: 
        :param sujeto_obligado: 
        :param aviso: 
        """
        
        super().__init__({
            'MesReportado': mes_reportado,
            'SujetoObligado': sujeto_obligado,
            'Aviso': aviso,
        })
        

class ArchivoType(XElement):
    def __init__(
            self,
            informe: InformeType | dict | Sequence[InformeType | dict],
    ): 
        """
        
        :param informe: 
        """
        
        super().__init__({
            'Informe': informe,
        })
        

class Archivo(ArchivoType):
    tag = '{http://www.uif.shcp.gob.mx/recepcion/ari}archivo'




from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class MovimientoECBFiscal(XElement):
    """
    http://www.sat.gob.mx/ecb
    Nodo requerido para expresar las operaciones a ser detalladas en el estado de cuenta bancario con RFC con efecto fiscal.
    """
    def __init__(
            self,
            fecha: datetime,
            descripcion: str,
            rfc_enajenante: str,
            importe: Decimal | int,
            referencia: str = None,
            moneda: str = None,
            saldo_inicial: Decimal | int = None,
            saldo_al_corte: Decimal | int = None,
    ): 
        """
        Nodo requerido para expresar las operaciones a ser detalladas en el estado de cuenta bancario con RFC con efecto fiscal.
        
        :param fecha: Atributo requerido únicamente para la expresión de la fecha (mes/día) de la operación realizada por el cuentahabiente.
        :param descripcion: Atributo requerido para precisar la descripción del bien o servicio cubierto por el cargo a la cuenta bancaria.
        :param rfc_enajenante: Atributo requerido para indicar el RFC del enajenante, sin el cual no se puede acreditar la compra de bienes o servicios por medio del estado de cuenta bancario.
        :param importe: Atributo requerido para indicar el importe de la operación realizada por el cuentahabiente por medio del instrumento bancario.
        :param referencia: Atributo opcional para indicar el número de referencia o autorización con el que se identifica la operación realizada por el cuentahabiente.
        :param moneda: Atributo opcional para indicar en que moneda se realizó la operación. Si no se especifica dato alguno, se entenderá que el importe está expresado en moneda nacional.
        :param saldo_inicial: Atributo opcional para indicar el saldo inicial del instrumento bancario del cuentahabiente.
        :param saldo_al_corte: Atributo opcional para indicar el saldo al corte del instrumento bancario del cuentahabiente.
        """
        
        super().__init__({
            'Fecha': fecha,
            'Descripcion': descripcion,
            'RFCenajenante': rfc_enajenante,
            'Importe': importe,
            'Referencia': referencia,
            'Moneda': moneda,
            'SaldoInicial': saldo_inicial,
            'SaldoAlCorte': saldo_al_corte,
        })
        

class MovimientoECB(XElement):
    """
    http://www.sat.gob.mx/ecb
    Nodo requerido para expresar las operaciones a ser detalladas en el estado de cuenta bancario que no cuentan con un RFC
    """
    def __init__(
            self,
            fecha: datetime,
            descripcion: str,
            importe: Decimal | int,
            referencia: str = None,
            moneda: str = None,
            saldo_inicial: Decimal | int = None,
            saldo_al_corte: Decimal | int = None,
    ): 
        """
        Nodo requerido para expresar las operaciones a ser detalladas en el estado de cuenta bancario que no cuentan con un RFC
        
        :param fecha: Atributo requerido únicamente para la expresión de la fecha (mes/día) de la operación realizada por el cuentahabiente.
        :param descripcion: Atributo requerido para precisar la descripción del bien o servicio cubierto por el cargo a la cuenta bancaria.
        :param importe: Atributo requerido para indicar el importe de la operación realizada por el cuentahabiente por medio del instrumento bancario.
        :param referencia: Atributo opcional para indicar el número de referencia o autorización con el que se identifica la operación realizada por el cuentahabiente.
        :param moneda: Atributo opcional para indicar en que moneda se realizó la operación. Si no se especifica dato alguno, se entenderá que el importe está expresado en moneda nacional.
        :param saldo_inicial: Atributo opcional para indicar el saldo inicial del instrumento bancario del cuentahabiente.
        :param saldo_al_corte: Atributo opcional para indicar el saldo al corte del instrumento bancario del cuentahabiente.
        """
        
        super().__init__({
            'Fecha': fecha,
            'Descripcion': descripcion,
            'Importe': importe,
            'Referencia': referencia,
            'Moneda': moneda,
            'SaldoInicial': saldo_inicial,
            'SaldoAlCorte': saldo_al_corte,
        })
        

class Movimientos(XElement):
    """
    http://www.sat.gob.mx/ecb
    Nodo requerido para enlistar los conceptos cubiertos por Estado de Cuenta Bancario.
    """
    def __init__(
            self,
            movimiento_ecb: MovimientoECB | dict = None,
            movimiento_ecbfiscal: MovimientoECBFiscal | dict = None,
    ): 
        """
        Nodo requerido para enlistar los conceptos cubiertos por Estado de Cuenta Bancario.
        
        :param movimiento_ecb: Nodo requerido para expresar las operaciones a ser detalladas en el estado de cuenta bancario que no cuentan con un RFC
        :param movimiento_ecbfiscal: Nodo requerido para expresar las operaciones a ser detalladas en el estado de cuenta bancario con RFC con efecto fiscal.
        """
        
        super().__init__({
            'MovimientoECB': movimiento_ecb,
            'MovimientoECBFiscal': movimiento_ecbfiscal,
        })
        

class EstadoDeCuentaBancario(CFDI):
    """
    Complemento al Comprobante Fiscal Digital (CFD) y Comprobante Fiscal Digital a través de Internet (CFDI) para integrar información aplicable al estado de cuenta bancario.
    """
    tag = '{http://www.sat.gob.mx/ecb}EstadoDeCuentaBancario'
    version = '1.0'
    
    def __init__(
            self,
            numero_cuenta: int,
            nombre_cliente: str,
            periodo: str,
            movimientos: Movimientos | dict,
            sucursal: str = None,
    ): 
        """
        Complemento al Comprobante Fiscal Digital (CFD) y Comprobante Fiscal Digital a través de Internet (CFDI) para integrar información aplicable al estado de cuenta bancario.
        
        :param numero_cuenta: Atributo requerido para indicar el número de cuenta del producto bancario.
        :param nombre_cliente: Atributo requerido para indicar el nombre del cuentahabiente de la institución bancaria.
        :param periodo: Atributo requerido para indicar periodo de los cargos o abonos al instrumento bancario.
        :param movimientos: Nodo requerido para enlistar los conceptos cubiertos por Estado de Cuenta Bancario.
        :param sucursal: Atributo opcional para indicar la sucursal a la cual pertenece el cuentahabiente.
        """
        
        super().__init__({
            'Version': self.version,
            'NumeroCuenta': numero_cuenta,
            'NombreCliente': nombre_cliente,
            'Periodo': periodo,
            'Movimientos': movimientos,
            'Sucursal': sucursal,
        })
        




from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class HorasExtra(XElement):
    """
    http://www.sat.gob.mx/nomina
    Nodo opcional para expresar información de las horas extras
    """
    def __init__(
            self,
            dias: int,
            tipo_horas: str,
            horas_extra: int,
            importe_pagado: Decimal | int,
    ): 
        """
        Nodo opcional para expresar información de las horas extras
        
        :param dias: Número de días en que el trabajador realizó horas extra en el periodo
        :param tipo_horas: Tipo de pago de las horas extra: dobles o triples
        :param horas_extra: Número de horas extra trabajadas en el periodo
        :param importe_pagado: Importe pagado por las horas extra
        """
        
        super().__init__({
            'Dias': dias,
            'TipoHoras': tipo_horas,
            'HorasExtra': horas_extra,
            'ImportePagado': importe_pagado,
        })
        

class Incapacidad(XElement):
    """
    http://www.sat.gob.mx/nomina
    Nodo opcional para expresar información de las incapacidades
    """
    def __init__(
            self,
            dias_incapacidad: Decimal | int,
            tipo_incapacidad: int,
            descuento: Decimal | int,
    ): 
        """
        Nodo opcional para expresar información de las incapacidades
        
        :param dias_incapacidad: Número de días que el trabajador se incapacitó en el periodo
        :param tipo_incapacidad: Razón de la incapacidad: Catálogo publicado en el portal del SAT en internet
        :param descuento: Monto del descuento por la incapacidad
        """
        
        super().__init__({
            'DiasIncapacidad': dias_incapacidad,
            'TipoIncapacidad': tipo_incapacidad,
            'Descuento': descuento,
        })
        

class Deduccion(XElement):
    """
    http://www.sat.gob.mx/nomina
    Nodo para expresar la información detallada de una deducción
    """
    def __init__(
            self,
            tipo_deduccion: int,
            clave: str,
            concepto: str,
            importe_gravado: Decimal | int,
            importe_exento: Decimal | int,
    ): 
        """
        Nodo para expresar la información detallada de una deducción
        
        :param tipo_deduccion: Clave agrupadora. Clasifica la deducción conforme al catálogo publicado en el portal del SAT en internet
        :param clave: Atributo requerido para la clave de deducción de nómina propia de la contabilidad de cada patrón, puede conformarse desde 3 hasta 15 caracteres
        :param concepto: Atributo requerido para la descripción del concepto de deducción
        :param importe_gravado: Atributo requerido, representa el importe gravado de un concepto de deducción
        :param importe_exento: Atributo requerido, representa el importe exento de un concepto de deducción
        """
        
        super().__init__({
            'TipoDeduccion': tipo_deduccion,
            'Clave': clave,
            'Concepto': concepto,
            'ImporteGravado': importe_gravado,
            'ImporteExento': importe_exento,
        })
        

class Deducciones(XElement):
    """
    http://www.sat.gob.mx/nomina
    Nodo opcional para expresar las deducciones aplicables
    """
    def __init__(
            self,
            total_gravado: Decimal | int,
            total_exento: Decimal | int,
            deduccion: Deduccion | dict | Sequence[Deduccion | dict],
    ): 
        """
        Nodo opcional para expresar las deducciones aplicables
        
        :param total_gravado: Atributo requerido para expresar el total de deducciones gravadas que se relacionan en el comprobante
        :param total_exento: Atributo requerido para expresar el total de deducciones exentas que se relacionan en el comprobante
        :param deduccion: Nodo para expresar la información detallada de una deducción
        """
        
        super().__init__({
            'TotalGravado': total_gravado,
            'TotalExento': total_exento,
            'Deduccion': deduccion,
        })
        

class Percepcion(XElement):
    """
    http://www.sat.gob.mx/nomina
    Nodo para expresar la información detallada de una percepción
    """
    def __init__(
            self,
            tipo_percepcion: int,
            clave: str,
            concepto: str,
            importe_gravado: Decimal | int,
            importe_exento: Decimal | int,
    ): 
        """
        Nodo para expresar la información detallada de una percepción
        
        :param tipo_percepcion: Clave agrupadora. Clasifica la percepción conforme al catálogo publicado en el portal del SAT en internet
        :param clave: Atributo requerido, representa la clave de percepción de nómina propia de la contabilidad de cada patrón, puede conformarse desde 3 hasta 15 caracteres
        :param concepto: Atributo requerido para la descripción del concepto de percepción
        :param importe_gravado: Atributo requerido, representa el importe gravado de un concepto de percepción
        :param importe_exento: Atributo requerido, representa el importe exento de un concepto de percepción
        """
        
        super().__init__({
            'TipoPercepcion': tipo_percepcion,
            'Clave': clave,
            'Concepto': concepto,
            'ImporteGravado': importe_gravado,
            'ImporteExento': importe_exento,
        })
        

class Percepciones(XElement):
    """
    http://www.sat.gob.mx/nomina
    Nodo opcional para expresar las percepciones aplicables
    """
    def __init__(
            self,
            total_gravado: Decimal | int,
            total_exento: Decimal | int,
            percepcion: Percepcion | dict | Sequence[Percepcion | dict],
    ): 
        """
        Nodo opcional para expresar las percepciones aplicables
        
        :param total_gravado: Atributo requerido para expresar el total de percepciones gravadas que se relacionan en el comprobante
        :param total_exento: Atributo requerido para expresar el total de percepciones exentas que se relacionan en el comprobante
        :param percepcion: Nodo para expresar la información detallada de una percepción
        """
        
        super().__init__({
            'TotalGravado': total_gravado,
            'TotalExento': total_exento,
            'Percepcion': percepcion,
        })
        

class Nomina(CFDI):
    """
    Complemento al Comprobante Fiscal Digital a través de Internet (CFDI) para el manejo de datos de Nómina.
    """
    tag = '{http://www.sat.gob.mx/nomina}Nomina'
    version = '1.1'
    
    def __init__(
            self,
            num_empleado: str,
            curp: str,
            tipo_regimen: int,
            fecha_pago: date,
            fecha_inicial_pago: date,
            fecha_final_pago: date,
            num_dias_pagados: Decimal | int,
            periodicidad_pago: str,
            registro_patronal: str = None,
            num_seguridad_social: str = None,
            departamento: str = None,
            clabe: int = None,
            banco: int = None,
            fecha_inicio_rel_laboral: date = None,
            antiguedad: int = None,
            puesto: str = None,
            tipo_contrato: str = None,
            tipo_jornada: str = None,
            salario_base_cot_apor: Decimal | int = None,
            riesgo_puesto: int = None,
            salario_diario_integrado: Decimal | int = None,
            percepciones: Percepciones | dict = None,
            deducciones: Deducciones | dict = None,
            incapacidades: Incapacidad | dict | Sequence[Incapacidad | dict] = None,
            horas_extras: HorasExtra | dict | Sequence[HorasExtra | dict] = None,
    ): 
        """
        Complemento al Comprobante Fiscal Digital a través de Internet (CFDI) para el manejo de datos de Nómina.
        
        :param num_empleado: Atributo requerido para expresar el número de empleado de 1 a 15 posiciones
        :param curp: Atributo requerido para la expresión de la CURP del trabajador
        :param tipo_regimen: Atributo requerido para la expresión de la clave del régimen por el cual se tiene contratado al trabajador, conforme al catálogo publicado en el portal del SAT en internet
        :param fecha_pago: Atributo requerido para la expresión de la fecha efectiva de erogación del gasto. Se expresa en la forma aaaa-mm-dd, de acuerdo con la especificación ISO 8601.
        :param fecha_inicial_pago: Atributo requerido para la expresión de la fecha inicial del pago. Se expresa en la forma aaaa-mm-dd, de acuerdo con la especificación ISO 8601.
        :param fecha_final_pago: Atributo requerido para la expresión de la fecha final del pago. Se expresa en la forma aaaa-mm-dd, de acuerdo con la especificación ISO 8601.
        :param num_dias_pagados: Atributo requerido para la expresión del número de días pagados
        :param periodicidad_pago: Forma en que se establece el pago del salario: diario, semanal, quincenal, catorcenal mensual, bimestral, unidad de obra, comisión, precio alzado, etc.
        :param registro_patronal: Atributo opcional para expresar el registro patronal a 20 posiciones máximo
        :param num_seguridad_social: Atributo opcional para la expresión del número de seguridad social aplicable al trabajador
        :param departamento: Atributo opcional para la expresión del departamento o área a la que pertenece el trabajador
        :param clabe: Atributo opcional para la expresión de la CLABE
        :param banco: Atributo opcional para la expresión del Banco conforme al catálogo, donde se realiza un depósito de nómina
        :param fecha_inicio_rel_laboral: Atributo opcional para expresar la fecha de inicio de la relación laboral entre el empleador y el empleado
        :param antiguedad: Número de semanas que el empleado ha mantenido relación laboral con el empleador
        :param puesto: Puesto asignado al empleado o actividad que realiza
        :param tipo_contrato: Tipo de contrato que tiene el trabajador: Base, Eventual, Confianza, Sindicalizado, a prueba, etc.
        :param tipo_jornada: Tipo de jornada que cubre el trabajador: Diurna, nocturna, mixta, por hora, reducida, continuada, partida, por turnos, etc.
        :param salario_base_cot_apor: Retribución otorgada al trabajador, que se integra por los pagos hechos en efectivo por cuota diaria, gratificaciones, percepciones, alimentación, habitación, primas, comisiones, prestaciones en especie y cualquiera otra cantidad o prestación que se entregue al trabajador por su trabajo, sin considerar los conceptos que se excluyen de conformidad con el Artículo 27 de la Ley del Seguro Social. (Se emplea para pagar las cuotas y aportaciones de Seguridad Social).
        :param riesgo_puesto: Clave conforme a la Clase en que deben inscribirse los patrones, de acuerdo a las actividades que desempeñan sus trabajadores, según lo previsto en el artículo 196 del Reglamento en Materia de Afiliación Clasificación de Empresas, Recaudación y Fiscalización. Catálogo publicado en el portal del SAT en internet
        :param salario_diario_integrado: El salario se integra con los pagos hechos en efectivo por cuota diaria, gratificaciones, percepciones, habitación, primas, comisiones, prestaciones en especie y cualquiera otra cantidad o prestación que se entregue al trabajador por su trabajo, de conformidad con el Art. 84 de la Ley Federal del Trabajo. (Se utiliza para el cálculo de las indemnizaciones).
        :param percepciones: Nodo opcional para expresar las percepciones aplicables
        :param deducciones: Nodo opcional para expresar las deducciones aplicables
        :param incapacidades: Nodo opcional para expresar las incapacidades aplicables
        :param horas_extras: Nodo opcional para expresar las horas extras aplicables
        """
        
        super().__init__({
            'Version': self.version,
            'NumEmpleado': num_empleado,
            'CURP': curp,
            'TipoRegimen': tipo_regimen,
            'FechaPago': fecha_pago,
            'FechaInicialPago': fecha_inicial_pago,
            'FechaFinalPago': fecha_final_pago,
            'NumDiasPagados': num_dias_pagados,
            'PeriodicidadPago': periodicidad_pago,
            'RegistroPatronal': registro_patronal,
            'NumSeguridadSocial': num_seguridad_social,
            'Departamento': departamento,
            'CLABE': clabe,
            'Banco': banco,
            'FechaInicioRelLaboral': fecha_inicio_rel_laboral,
            'Antiguedad': antiguedad,
            'Puesto': puesto,
            'TipoContrato': tipo_contrato,
            'TipoJornada': tipo_jornada,
            'SalarioBaseCotApor': salario_base_cot_apor,
            'RiesgoPuesto': riesgo_puesto,
            'SalarioDiarioIntegrado': salario_diario_integrado,
            'Percepciones': percepciones,
            'Deducciones': deducciones,
            'Incapacidades': incapacidades,
            'HorasExtras': horas_extras,
        })
        




from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class DocumentoRelacionado(XElement):
    """
    http://www.sat.gob.mx/IngresosHidrocarburos10
    Nodo requerido para expresar la información del documento relacionado al ingreso.
    """
    def __init__(
            self,
            folio_fiscal_vinculado: str,
            fecha_folio_fiscal_vinculado: date,
            mes: str,
    ): 
        """
        Nodo requerido para expresar la información del documento relacionado al ingreso.
        
        :param folio_fiscal_vinculado: Atributo requerido para expresar el folio fiscal del CFDI expedido por el operador del consorcio al FMP.
        :param fecha_folio_fiscal_vinculado: Atributo requerido para expresar la fecha del CFDI expedido por el operador del consorcio al FMP. Se expresa en la forma aaaa-mm-dd, de acuerdo con la especificación ISO 8601.
        :param mes: Atributo requerido para expresar el mes que corresponda al CFDI expedido por el operador del consorcio al FMP.
        """
        
        super().__init__({
            'FolioFiscalVinculado': folio_fiscal_vinculado,
            'FechaFolioFiscalVinculado': fecha_folio_fiscal_vinculado,
            'Mes': mes,
        })
        

class IngresosHidrocarburos(CFDI):
    """
    Complemento para incorporar la información sobre Ingresos atribuibles a los Integrantes de un consorcio derivados de la contraprestación de un contrato de exploración o extracción de hidrocarburos.
    """
    tag = '{http://www.sat.gob.mx/IngresosHidrocarburos10}IngresosHidrocarburos'
    version = '1.0'
    
    def __init__(
            self,
            numero_contrato: str,
            contraprestacion_pagada_operador: Decimal | int,
            porcentaje: Decimal | int,
            documento_relacionado: DocumentoRelacionado | dict | Sequence[DocumentoRelacionado | dict],
    ): 
        """
        Complemento para incorporar la información sobre Ingresos atribuibles a los Integrantes de un consorcio derivados de la contraprestación de un contrato de exploración o extracción de hidrocarburos.
        
        :param numero_contrato: Atributo requerido para expresar el número de contrato asignado por la Comisión Nacional de Hidrocarburos con el cual se encuentra vinculado el ingreso.
        :param contraprestacion_pagada_operador: Atributo requerido para precisar el importe total de las contraprestaciones pagadas al operador del consorcio.
        :param porcentaje: Atributo requerido para expresar el porcentaje que ampara el CFDI que emite cada integrante del consorcio al operador, respecto del total de las contraprestaciones entregadas al operador del consorcio por el FMP.
        :param documento_relacionado: Nodo requerido para expresar la información del documento relacionado al ingreso.
        """
        
        super().__init__({
            'Version': self.version,
            'NumeroContrato': numero_contrato,
            'ContraprestacionPagadaOperador': contraprestacion_pagada_operador,
            'Porcentaje': porcentaje,
            'DocumentoRelacionado': documento_relacionado,
        })
        




from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class Beneficiario(XElement):
    """
    http://www.sat.gob.mx/esquemas/retencionpago/1/pagosaextranjeros
    Nodo opcional para precisar la información del representante para efectos fiscales en México
    """
    def __init__(
            self,
            rfc: str,
            curp: str,
            nom_den_raz_soc_b: str,
            concepto_pago: str,
            descripcion_concepto: str,
    ): 
        """
        Nodo opcional para precisar la información del representante para efectos fiscales en México
        
        :param rfc: Atributo requerido para expresar la clave del registro federal de contribuyentes del representante legal en México
        :param curp: Atributo requerido para la expresión de la CURP del representante legal
        :param nom_den_raz_soc_b: Atributo requerido para expresar el nombre, denominación o razón social del contribuyente en México
        :param concepto_pago: Atributo requerido para expresar el tipo de contribuyente sujeto a la retención, conforme al catálogo.
        :param descripcion_concepto: Atributo requerido para expresar la descripción de la definición del pago del residente en el extranjero
        """
        
        super().__init__({
            'RFC': rfc,
            'CURP': curp,
            'NomDenRazSocB': nom_den_raz_soc_b,
            'ConceptoPago': concepto_pago,
            'DescripcionConcepto': descripcion_concepto,
        })
        

class NoBeneficiario(XElement):
    """
    http://www.sat.gob.mx/esquemas/retencionpago/1/pagosaextranjeros
    Nodo opcional para expresar la información del residente extranjero efectivo del cobro
    """
    def __init__(
            self,
            pais_de_resid_para_efec_fisc: str,
            concepto_pago: str,
            descripcion_concepto: str,
    ): 
        """
        Nodo opcional para expresar la información del residente extranjero efectivo del cobro
        
        :param pais_de_resid_para_efec_fisc: Atributo requerido para expresar la clave del país de residencia del extranjero, conforme al catálogo.
        :param concepto_pago: Atributo requerido para expresar el tipo contribuyente sujeto a la retención, conforme al catálogo.
        :param descripcion_concepto: Atributo requerido para expresar la descripción de la definición del pago del residente en el extranjero
        """
        
        super().__init__({
            'PaisDeResidParaEfecFisc': pais_de_resid_para_efec_fisc,
            'ConceptoPago': concepto_pago,
            'DescripcionConcepto': descripcion_concepto,
        })
        

class Pagosaextranjeros(CFDI):
    """
    Complemento para expresar los pagos que se realizan a residentes en el extranjero
    """
    tag = '{http://www.sat.gob.mx/esquemas/retencionpago/1/pagosaextranjeros}Pagosaextranjeros'
    version = '1.0'
    
    def __init__(
            self,
            es_benef_efect_del_cobro: str,
            no_beneficiario: NoBeneficiario | dict = None,
            beneficiario: Beneficiario | dict = None,
    ): 
        """
        Complemento para expresar los pagos que se realizan a residentes en el extranjero
        
        :param es_benef_efect_del_cobro: Atributo requerido para expresar si el beneficiario del pago es la misma persona que retiene
        :param no_beneficiario: Nodo opcional para expresar la información del residente extranjero efectivo del cobro
        :param beneficiario: Nodo opcional para precisar la información del representante para efectos fiscales en México
        """
        
        super().__init__({
            'Version': self.version,
            'EsBenefEfectDelCobro': es_benef_efect_del_cobro,
            'NoBeneficiario': no_beneficiario,
            'Beneficiario': beneficiario,
        })
        




import pickle
import requests
import urllib3

from .utils import get_post_form, generate_token
from .. import Signer, __version__

CONSTANCIA_URL = 'https://rfcampc.siat.sat.gob.mx/PTSC/IdcSiat/IdcGeneraConstancia.jsf'
USER_AGENT = __version__.__user_agent__

DEFAULT_HEADERS = {
    'accept': 'text/html,application/xhtml+xml,application/xml',
    'pragma': 'no-cache',
    'cache-control': 'no-cache',
    'user-agent': USER_AGENT
}


def debug_response(res):
    print('-- RESPONSE DEBUG --')
    print(res.status_code)
    for k, v in res.request.headers.items():
        print(k, v)
    print(res.request.body)
    print('results')
    print(res.text)
    print('-- RESPONSE DEBUG END --')


class SATSession:
    def __init__(self, signer: Signer):
        urllib3.util.ssl_.DEFAULT_CIPHERS += ':HIGH:!DH'
        self.signer = signer
        self.session = requests.session()

    def login(self):
        LOGIN_URL = 'https://loginda.siat.sat.gob.mx/nidp/app/login?id=fiel'
        LOGIN_URL_ORIGIN = 'https://loginda.siat.sat.gob.mx'

        res = self.session.get(
            url=LOGIN_URL,
            headers=DEFAULT_HEADERS
        )
        assert res.status_code == 200

        action, data = get_post_form(res)
        res = self.session.post(
            url=action,
            headers=DEFAULT_HEADERS | {
                'origin': LOGIN_URL_ORIGIN,
                'referer': LOGIN_URL,
            },
            data=data
        )
        assert res.status_code == 200

        action, data = get_post_form(res, id='certform')
        res = self.session.post(
            url=action,
            headers=DEFAULT_HEADERS | {
                'origin': LOGIN_URL_ORIGIN,
                'referer': LOGIN_URL,
            },
            data=data | {
                'token': generate_token(self.signer, code=data['guid']),
                'fert': self.signer.certificate.get_notAfter()[2:].decode(),
            }
        )
        assert res.status_code == 200

        return res

    def home_page(self):
        return self.session.get(
            url='https://loginda.siat.sat.gob.mx/nidp/app?sid=0',
            headers=DEFAULT_HEADERS
        )

    def logout(self):
        return self.session.get(
            url='https://loginda.siat.sat.gob.mx/nidp/app/logout',
            headers=DEFAULT_HEADERS | {
                'referer': 'https://loginda.siat.sat.gob.mx/nidp/app?sid=0'
            },
            allow_redirects=False
        )

    def declaraciones_provisionales_login(self):
        res = self.session.get(
            url='https://ptscdecprov.clouda.sat.gob.mx/',
            headers=DEFAULT_HEADERS,
            allow_redirects=True
        )
        assert res.status_code == 200

        action, data = get_post_form(res)
        res = self.session.post(
            url=action,
            headers=DEFAULT_HEADERS | {
                'content-type': 'application/x-www-form-urlencoded',
                'origin': 'https://loginda.siat.sat.gob.mx',
                'referer': 'https://loginda.siat.sat.gob.mx/',
            },
            data=data
        )
        return res

    def save_session(self, target):
        pickle.dump(self.session.cookies, target)

    def load_session(self, source):
        self.session.cookies.update(pickle.load(source))



import enum
from collections.abc import Sequence, Mapping


class ScalarMap(dict):
    pass


def iterate(item):
    if isinstance(item, str | bytes | ScalarMap):
        return [item]
    if isinstance(item, Mapping):
        return item.values()
    if isinstance(item, Sequence):
        return item
    if item is None:
        return []
    return [item]


class StrEnum(str, enum.Enum):
    def __str__(self):
        return str(self.value)




# -*- coding: utf-8 -*-
import base64

from OpenSSL import crypto
from OpenSSL.crypto import X509
from cryptography import x509
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import padding, rsa
from cryptography.hazmat.primitives.serialization import Encoding, pkcs12, PrivateFormat, BestAvailableEncryption, NoEncryption, PublicFormat, load_der_private_key

from .certificate import Certificate
from ..exceptions import CFDIError


class Signer(Certificate):
    def __init__(self, certificate: X509, key: rsa.RSAPrivateKey, check=True):
        super().__init__(certificate)
        self.key = key

        if check:
            res = _compare_public_keys(self.key.public_key(), self.public_key())
            if not res:
                raise CFDIError("Private Key does not match certificate")

    @classmethod
    def load(cls, certificate: bytes, key: bytes, password: str | bytes = None, check=True) -> 'Signer':
        if isinstance(password, str):
            password = password.encode()

        return cls(
            # certificate=x509.load_der_x509_certificate(certificate),
            certificate=crypto.load_certificate(crypto.FILETYPE_ASN1, certificate),
            key=load_der_private_key(
                data=key,
                password=password
            ),
            check=check
        )

    @classmethod
    def load_pkcs12(cls, data: bytes, password: str | bytes = None) -> 'Signer':
        if isinstance(password, str):
            password = password.encode()

        key, certificate, _ = pkcs12.load_key_and_certificates(data=data, password=password)
        if certificate is None:
            raise CFDIError("Certificate is missing")

        return cls(
            certificate=crypto.X509.from_cryptography(certificate),
            key=key,
            check=False  # pcks12 allready checks
        )

    def _sign(self, data, algorithm) -> str:
        signature = self.key.sign(
            data=data,
            padding=padding.PKCS1v15(),
            algorithm=algorithm
        )

        return base64.b64encode(
            signature
        ).decode()

    def sign_sha1(self, data) -> str:
        return self._sign(
            data=data,
            algorithm=hashes.SHA1()
        )

    def sign_sha256(self, data) -> str:
        return self._sign(
            data=data,
            algorithm=hashes.SHA256()
        )

    def key_bytes(self, password: str | bytes = None) -> bytes:
        if isinstance(password, str):
            password = password.encode()

        return self.key.private_bytes(
            encoding=Encoding.DER,
            format=PrivateFormat.PKCS8,
            encryption_algorithm=BestAvailableEncryption(password) if password else NoEncryption()
        )

    def pcks12_bytes(self, password: str | bytes = None) -> bytes:
        if isinstance(password, str):
            password = password.encode()

        return pkcs12.serialize_key_and_certificates(
            name=self.rfc.encode(),
            key=self.key,
            cert=self.certificate.to_cryptography(),
            cas=None,
            encryption_algorithm=BestAvailableEncryption(password) if password else NoEncryption()
        )

    def decrypt(self, data: bytes):
        return self.key.decrypt(
            ciphertext=data,
            padding=padding.PKCS1v15()
        )


def _compare_public_keys(public_key_a, public_key_b):
    def key_bytes(k):
        return k.public_bytes(
            encoding=Encoding.DER,
            format=PublicFormat.SubjectPublicKeyInfo
        )

    return key_bytes(public_key_a) == key_bytes(public_key_b)



from datetime import datetime

from ...cfdi import CFDI, XElement, MEXICO_TZ
from ...models.signer import Signer


class TimbreFiscalDigital(CFDI):
    """
    Complemento requerido para el Timbrado Fiscal Digital que da validez al Comprobante fiscal digital por Internet.
    """
    tag = '{http://www.sat.gob.mx/TimbreFiscalDigital}TimbreFiscalDigital'
    version = '1.1'

    def __init__(
            self,
            proveedor: Signer,
            uuid: str,
            fecha_timbrado: datetime,
            sello_cfd: str,
            leyenda: str = None,
    ):
        """
        Complemento requerido para el Timbrado Fiscal Digital que da valides a un Comprobante Fiscal Digital.

        :param uuid: Atributo requerido para expresar los 36 caracteres del UUID de la transacción de timbrado
        :param fecha_timbrado: Atributo requerido para expresar la fecha y hora de la generación del timbre
        :param sello_cfd: Atributo requerido para contener el sello digital del comprobante fiscal, que será timbrado. El sello deberá ser expresado cómo una cadena de texto en formato Base 64.
        :param no_certificado_sat: Atributo requerido para expresar el número de serie del certificado del SAT usado para el Timbre
        :param sello_sat: Atributo requerido para contener el sello digital del Timbre Fiscal Digital, al que hacen referencia las reglas de resolución miscelánea aplicable. El sello deberá ser expresado cómo una cadena de texto en formato Base 64.
        :return: objeto CFDI
        """

        super().__init__({
            'Version': self.version,
            'UUID': uuid,
            'FechaTimbrado': fecha_timbrado or datetime.now(tz=MEXICO_TZ).replace(tzinfo=None),
            'RfcProvCertif': proveedor.rfc_pac,
            'SelloCFD': sello_cfd,
            'NoCertificadoSAT': proveedor.certificate_number,
            'SelloSAT': '',
            'Leyenda': leyenda,
        })

        self['SelloSAT'] = proveedor.sign_sha256(
            self.cadena_original().encode()
        )



import os
import setuptools

about = {}
current_dir = os.path.abspath(os.path.dirname(__file__))
package_dir = 'satcfdi'

with open(os.path.join(current_dir, package_dir, "__version__.py"), "r", encoding="utf-8") as f:
    exec(f.read(), about)

package = about["__package__"]

packages = setuptools.find_packages(
    exclude=["contrib", "docs", "tests"],
)

setuptools.setup(
    name=package,  # installation
    version=about["__version__"],
    author=about["__author__"],
    author_email=about["__author__"],
    description=about["__description__"],
    long_description=open('readme.rst', 'r', encoding='utf-8').read(),
    long_description_content_type="text/x-rst",
    url=about["__url__"],
    project_urls={
        "Documentation": about["__docs_url__"],
        "Source": about["__url__"],
        # "Changelog": about["__change_log_url__"],
    },
    license=about["__license__"],
    keywords=[
        'cfdi',
        'sat',
        'facturación',
        'comprobante',
        'retenciones',
        'nómina',
        'pagos',
        'carta porte',
        'contabilidad',
        'e-invoicing',
        'DIOT'
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: Spanish",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11"
    ],
    python_requires='>=3.10',
    package_dir={package: package_dir},
    packages=packages,
    package_data={
        package: [
            "diot/*",
            "transform/pdf_templates/*",
            "transform/schemas/*",
            "transform/schemas/*/*",
            "transform/schemas/*/*/*",
            "transform/schemas/*/*/*/*",
            "transform/schemas/*/*/*/*/*",
            "transform/schemas/*/*/*/*/*/*",
            "transform/schemas/*/*/*/*/*/*/*",
            "transform/CertsProd.zip",
            "pacs/sat_templates/*",
        ],
    },
    install_requires=[
        'jinja2 >= 3.0.0',
        'lxml >= 4.9.0',
        'weasyprint >= 57.0',
        'requests >= 2.0.0',
        'cryptography >= 38.0.0',
        'pytz >= 2022.5',
        'xlsxwriter >= 3.0.0',
        'pyOpenSSL >= 22.0.0',
        'qrcode >= 7.3.0',
        'tabulate >= 0.9.0',
        'packaging >= 21.0',
        'beautifulsoup4 > 4.11.0'
    ],
    # tests_require=test_deps,
    entry_points={
        "console_scripts": [
            f"{package}={package}.cli.main:main"
        ],
    },
    # extras={
    #     'test': test_deps,
    # },
    extras_require={
        'test': [
            'coverage',
            'pytest',
        ],
        'docs': [
            'Sphinx >= 5.3.0',
            'sphinx-rtd-theme >= 1.1.1'
        ]
    }
)



from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class Intereseshipotecarios(CFDI):
    """
    Complemento para expresar la información sobre Intereses reales deducibles por crédito hipotecarios
    """
    tag = '{http://www.sat.gob.mx/esquemas/retencionpago/1/intereseshipotecarios}Intereseshipotecarios'
    version = '1.0'
    
    def __init__(
            self,
            credito_de_inst_financ: str,
            saldo_insoluto: Decimal | int,
            prop_deduc_del_credit: Decimal | int = None,
            mont_tot_int_nominales_dev: Decimal | int = None,
            mont_tot_int_nominales_dev_y_pag: Decimal | int = None,
            mont_tot_int_real_pag_deduc: Decimal | int = None,
            num_contrato: str = None,
    ): 
        """
        Complemento para expresar la información sobre Intereses reales deducibles por crédito hipotecarios
        
        :param credito_de_inst_financ: Atributo requerido para expresar si el crédito otorgado fue por institución financiera
        :param saldo_insoluto: Atributo requerido para expresar el saldo insoluto al 31 de diciembre del ejercicio inmediato anterior o fecha de contratación si se llevo a cabo en el ejercicio en curso
        :param prop_deduc_del_credit: Atributo opcional que expresa la proporción deducible del crédito aplicable sobre los intereses reales devengados y pagados
        :param mont_tot_int_nominales_dev: Atributo opcional que expresa el monto total de intereses nominales devengados
        :param mont_tot_int_nominales_dev_y_pag: Atributo opcional que expresa el monto total de intereses nominales devengados y pagados
        :param mont_tot_int_real_pag_deduc: Atributo opcional que expresa el monto total de intereses reales pagados deducibles
        :param num_contrato: Atributo opcional que expresa el número de contrato del crédito hipotecario
        """
        
        super().__init__({
            'Version': self.version,
            'CreditoDeInstFinanc': credito_de_inst_financ,
            'SaldoInsoluto': saldo_insoluto,
            'PropDeducDelCredit': prop_deduc_del_credit,
            'MontTotIntNominalesDev': mont_tot_int_nominales_dev,
            'MontTotIntNominalesDevYPag': mont_tot_int_nominales_dev_y_pag,
            'MontTotIntRealPagDeduc': mont_tot_int_real_pag_deduc,
            'NumContrato': num_contrato,
        })
        




from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ...cfdi import CFDI, XElement


class Planesderetiro(CFDI):
    """
    Complemento para expresar la información sobre los planes personales de retiro
    """
    tag = '{http://www.sat.gob.mx/esquemas/retencionpago/1/planesderetiro}Planesderetiro'
    version = '1.0'
    
    def __init__(
            self,
            sistema_financ: str,
            mont_int_reales_deveng_anioo_inm_ant: Decimal | int,
            hubo_retiros_anio_inm_ant_per: str,
            hubo_retiros_anio_inm_ant: str,
            mont_tot_aport_anio_inm_anterior: Decimal | int = None,
            mont_tot_retirado_anio_inm_ant_per: Decimal | int = None,
            mont_tot_exent_retirado_anio_inm_ant: Decimal | int = None,
            mont_tot_exedente_anio_inm_ant: Decimal | int = None,
            mont_tot_retirado_anio_inm_ant: Decimal | int = None,
    ): 
        """
        Complemento para expresar la información sobre los planes personales de retiro
        
        :param sistema_financ: Atributo requerido para expresar si los planes personales de retiro son del sistema financiero
        :param mont_int_reales_deveng_anioo_inm_ant: Atributo requerido para expresar el monto de los intereses reales devengados o percibidos durante el año inmediato anterior de los planes personales de retiro
        :param hubo_retiros_anio_inm_ant_per: Atributo requerido para expresar si se realizaron retiros de recursos invertidos y sus rendimientos en el ejercicio inmediato anterior antes de cumplir los requisitos de permanencia
        :param hubo_retiros_anio_inm_ant: Atributo requerido que expresa si se realizaron retiros en el ejercicio inmediato anterior
        :param mont_tot_aport_anio_inm_anterior: Atributo opcional que expresa el monto total de las aportaciones actualizadas en el año inmediato anterior de los planes personales de retiro
        :param mont_tot_retirado_anio_inm_ant_per: Atributo opcional que expresa el monto total del retiro realizado antes de cumplir con los requisitos de permanencia
        :param mont_tot_exent_retirado_anio_inm_ant: Atributo opcional que expresa el monto total exento del retiro realizado en el ejercicio inmediato anterior
        :param mont_tot_exedente_anio_inm_ant: Atributo opcional que expresa el monto total excedente del monto exento del retiro realizado en el ejercicio inmediato anterior
        :param mont_tot_retirado_anio_inm_ant: Atributo opcional que expresa el monto total del retiro realizado en el ejercicio inmediato anterior
        """
        
        super().__init__({
            'Version': self.version,
            'SistemaFinanc': sistema_financ,
            'MontIntRealesDevengAniooInmAnt': mont_int_reales_deveng_anioo_inm_ant,
            'HuboRetirosAnioInmAntPer': hubo_retiros_anio_inm_ant_per,
            'HuboRetirosAnioInmAnt': hubo_retiros_anio_inm_ant,
            'MontTotAportAnioInmAnterior': mont_tot_aport_anio_inm_anterior,
            'MontTotRetiradoAnioInmAntPer': mont_tot_retirado_anio_inm_ant_per,
            'MontTotExentRetiradoAnioInmAnt': mont_tot_exent_retirado_anio_inm_ant,
            'MontTotExedenteAnioInmAnt': mont_tot_exedente_anio_inm_ant,
            'MontTotRetiradoAnioInmAnt': mont_tot_retirado_anio_inm_ant,
        })
        

